// #############################################################################
// *****************************************************************************
//                  Copyright ( c ) 2008, Advantech Automation Corp.
//      THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY
//               INFORMATION WHICH IS THE PROPERTY OF ADVANTECH AUTOMATION CORP.
//
//    ANY DISCLOSURE, USE, OR REPRODUCTION, WITHOUT WRITTEN AUTHORIZATION FROM
//               ADVANTECH AUTOMATION CORP., IS STRICTLY PROHIBITED.
// *****************************************************************************
// #############################################################################
/***
 * File:         baud.c
 * Created:      2008-10-20
 * Revision:       1.00 
 *
 * Description:      simple driver test: change the bit rate registers with ioctl()
 *         first argument can be the device name -- else it uses advcan0
 *         if a second arg is given, it is used as new bit rate
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <devctl.h>
#include <fcntl.h>
#include "advcan.h"

#define STDDEV "advcan1"

int set_bitrate(
   int can_fd, /* device descriptor */
   int baud    /* bit rate */
   )
{
   Config_par_t  cfg;
   volatile Command_par_t cmd;

   cmd.cmd = CMD_STOP;
   ioctl(can_fd, CAN_IOCTL_COMMAND, &cmd);

   cfg.target = CONF_TIMING; 
   cfg.val1   = baud;
   ioctl(can_fd, CAN_IOCTL_CONFIG, &cfg);

   cmd.cmd = CMD_START;
   ioctl(can_fd, CAN_IOCTL_COMMAND, &cmd);
   return 0;
}

int main(int argc,char **argv)
{
   int can_fd;
   char device[40];
   int newbaud = 125;
   canmsg_t txmsg;
   int ret;

   printf("usage: %s [dev] [bit_rate]\n", argv[0]);
   printf("e.g.:\n");
   printf("./baud advcan0 125\n");

   if(argc > 1) 
   {
      sprintf(device, "/dev/%s", argv[1]);
   }
   else 
   {
      sprintf(device, "/dev/%s", STDDEV);
   }
   printf("using CAN device %s\n", device);
    
   if(( can_fd = open(device, O_RDWR )) < 0 ) 
   {
      printf("Error opening CAN device %s\n", device);
      exit(1);
   }

   if(argc == 3) 
   {
       newbaud = atoi(argv[2]);
   }

   printf("set baudrate to %d Kbit/s\n", newbaud);
   set_bitrate(can_fd, newbaud);

   close(can_fd);
   return 0;
}

