
// #############################################################################
// *****************************************************************************
//                  Copyright ( c ) 2008, Advantech Automation Corp.
//      THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY
//               INFORMATION WHICH IS THE PROPERTY OF ADVANTECH AUTOMATION CORP.
//
//    ANY DISCLOSURE, USE, OR REPRODUCTION, WITHOUT WRITTEN AUTHORIZATION FROM
//               ADVANTECH AUTOMATION CORP., IS STRICTLY PROHIBITED.
// *****************************************************************************
// #############################################################################
/***
 * File:      filter.c
 * Created:      2008-10-20
 * Revision:      1.00 
 *
 * Description:      simple driver test: set the acceptance code and mask with ioctl()
*/
/*****************************************************************
   Example for extended message format

          Bits
    mask  31 30 .....           4  3  2  1  0
    code
    -------------------------------------------
    ID    28 27 .....           1  0  R  +--+-> unused
                                      T
                                      R

    acccode =  (id << 3) + (rtr << 2) 
*******************************************************************

   Example for base message format

          Bits
    mask  31 30 .....           23 22 21 20 ... 0
    code
    -------------------------------------------
    ID    10  9.....                1  0  R  +--+-> unused
                                          T
                                          R
********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <devctl.h>
#include <unistd.h>
#include "advcan.h"

#define STDDEV "advcan0"
/***********************************************************************
*
* set_accmask - sets the CAN acceptance and mask register
*
*
* Changing these registers only possible in Reset mode.
*
* RETURN:
*
***************************************************************************/

int set_accmask(
   int fd,      /* device descriptor */
   int newcode,
   int newmask
   )
{
   Config_par_t  cfg;
   volatile Command_par_t cmd;

   cmd.cmd = CMD_STOP;
   ioctl(fd, CAN_IOCTL_COMMAND, &cmd);

   cfg.target = CONF_ACC; 
   cfg.val1   = newmask;
   cfg.val2   = newcode;
   ioctl(fd, CAN_IOCTL_CONFIG, &cfg);

   cmd.cmd = CMD_START;
   ioctl(fd, CAN_IOCTL_COMMAND, &cmd);
   return 0;
}


int set_acccode(
   int fd,      /* device descriptor */
   int newcode
   )
{
   Config_par_t  cfg;
   volatile Command_par_t cmd;
   cmd.cmd = CMD_STOP;
   ioctl(fd, CAN_IOCTL_COMMAND, &cmd);

   cfg.target = CONF_ACCC;
   cfg.val1   = newcode;
   ioctl(fd, CAN_IOCTL_CONFIG, &cfg);
   
   //set single filter  
   cfg.target = CONF_ACC_FILTER;
   cfg.val1   = 1;
   ioctl(fd, CAN_IOCTL_CONFIG, &cfg);

   cmd.cmd = CMD_START;
   ioctl(fd, CAN_IOCTL_COMMAND, &cmd);
   return 0;
}

int main(int argc,char **argv)
{
   int fd0;
   int fd1;
   int newcode  = 0;
   int sent;
   canmsg_t tx[256];
   canmsg_t rx[256];
   int got;
   int status;
   int i;
   int j;

   printf("acccode =  (id << 3) + (rtr << 2) for extend message\n"); 
   printf("acccode =  (id << 21) + (rtr << 20)  for base  message\n"); 
   printf("using /dev/advcan0 device to accept message\n"); 
   printf("using /dev/advcan1 to transmit message\n");

   printf("set accept code to only accept the message which id = 10 and rtr = 0\n");
    
   if(( fd0 = open("/dev/advcan0", O_RDWR )) < 0 ) 
   {
      fprintf(stderr,"Error opening CAN device can0\n");
      exit(1);
   }

   if(( fd1 = open("/dev/advcan1", O_RDWR )) < 0 ) 
   {
      fprintf(stderr,"Error opening CAN device can1\n");
      exit(1);
   }
   //only accept the message which id = 10 and rtr = 0
   //to learn about how to set the accept code,please read the manual for detail
   newcode = (10 << 21) + (0 << 20);
   printf("change acc_code to 0x%x\n", newcode);
   set_accmask(fd0, newcode, 0xfffff);

   sleep(1);  
   if( fork()==0 )
   {  
      while(1) 
      {
         got=read(fd0, &rx, sizeof(canmsg_t));
         if( got > 0) 
         {
            got = got / sizeof (canmsg_t);
            for(i = 0; i < got; i++) 
            {
               printf("Received with ret=%d: %12lu.%06lu id=%ld\n",
               got, 
               rx[i].timestamp.tv_sec,
               rx[i].timestamp.tv_usec,
               rx[i].id);
               printf("\tlen=%d msg=", rx[i].length);
               for(j = 0; j < rx[i].length; j++) 
               {
                  printf(" %02x", rx[i].data[j]);
               }
               printf(" flags=0x%02x\n", rx[i].flags );
               fflush(stdout);
            }
         } 
         else 
         {
			sleep(1);
         }
      }
      close(fd0);
   }
   else
   {
      for(i = 0; i < 20; i++) 
      {
         sprintf( (char*)tx[i].data,"msg%d",i);
         printf("sending '%s'\n", tx[i].data );
         tx[i].flags =0;// MSG_EXT;  
         tx[i].length = strlen((char*)tx[i].data);  
         tx[i].id=i;
      }
      sent = write(fd1, tx, 20*sizeof(canmsg_t)); 
      if(sent <= 0) 
      {
         perror("sent");
      }
      close(fd1);
   }
   wait(&status);
   return 0;
}
